### Takes a bunch of filenames, usually generated by make
### Generates an R file that:
###	loads all .env files
### Puts .?sv an .xls file names an a variable called input_files
###	runs commands in .R files (obeying START and END commands)
###	saves image to a .env file
### Make should pipe output to a .out file,
### and provide its name as first argument.
 
use strict;
 
my @env;
my @R;
my @sv;
 
my $target = shift(@ARGV);
die "Illegal target $target" unless $target =~ s/out$/env/;
 
foreach(@ARGV){
	push @env, $_ if /env$/;
	push @R, $_ if /R$/;
	push @sv, "\"$_\"" if /sv$/ or /xls$/;
}
 
foreach(@env){
	print "load('$_')\n";
}
print "input_files <- c(";
print join ", ", @sv;
print ")\n";
print "pdf()\n# End RR preface\n";
 
foreach my $f (@R){
	my $text;
	open (INF, $f);
	while(<INF>){
		if (/^END/){
			print STDERR ("END at line $. in $f\n");
			last;
		}
		$text .= $_;
		if (/^START/){
			print STDERR ("START at line $. in $f\n");
			$text = "";
		}
	}
	print $text;
}
 
print "# Begin RR postscript\nsave.image('$target')\n";

